#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 PanXu, Inc. All Rights Reserved
#
"""
预训练词向量载入器

Authors: panxu(panxu@baidu.com)
Date:    2020/06/24 11:35:00
"""

from .pretrained_word_embedding_loader import PretrainedWordEmbeddingLoader
from .general_pretrained_word_embedding_loader import GeneralPretrainedWordEmbeddingLoader
from .glove import GloveLoader
