#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 Baidu.com, Inc. All Rights Reserved
#
"""
实例

Authors: panxu(panxu@baidu.com)
Date:    2020/05/13 15:04:00
"""

from collections import OrderedDict


class Instance(OrderedDict):
    """
    Instance 类型
    """

    pass